@@ -1,5 +1,5 @@
-#![cfg_attr(not(ci_arti_nightly), allow(unknown_lints))]
-// @@ begin test lint list maintained by maint/add_warning @@
+// @@ begin example lint list maintained by maint/add_warning @@
+#![allow(unknown_lints)] // @@REMOVE_WHEN(ci_arti_nightly)
 #![allow(clippy::bool_assert_comparison)]
 #![allow(clippy::clone_on_copy)]
 #![allow(clippy::dbg_macro)]
@@ -11,7 +11,7 @@
 #![allow(clippy::unchecked_duration_subtraction)]
 #![allow(clippy::useless_vec)]
 #![allow(clippy::needless_pass_by_value)]
-//! <!-- @@ end test lint list maintained by maint/add_warning @@ -->
+//! <!-- @@ end example lint list maintained by maint/add_warning @@ -->
 
 // This example showcase how to use the trait IsolationHelper to build complex isolation rules.
 // For most usages, using a combination of TorClient::isolated_client and IsolationToken should
