@@ -11,7 +11,6 @@ use session::ArtiRpcSession;
 use std::{io::Result as IoResult, sync::Arc};
 use tor_config::{define_list_builder_helper, impl_standard_builder, ConfigBuildError};
 use tor_config_path::CfgPathResolver;
-use tor_rpc_connect::auth::RpcAuth;
 use tracing::{debug, info};
 
 use arti_client::TorClient;
@@ -147,13 +146,12 @@ pub(crate) async fn launch_rpc_mgr<R: Runtime>(
     resolver: &CfgPathResolver,
     mistrust: &Mistrust,
     client: TorClient<R>,
-) -> Result<Option<(Arc<RpcMgr>, RpcStateSender)>> {
+) -> Result<Option<RpcProxySupport>> {
     if !cfg.enable {
         return Ok(None);
     }
     let (rpc_state, rpc_state_sender) = RpcVisibleArtiState::new();
 
-    // TODO RPC: there should be an error return instead.
     let rpc_mgr = RpcMgr::new(move |auth| ArtiRpcSession::new(auth, &client, &rpc_state))?;
     // Register methods. Needed since TorClient is generic.
     //
@@ -176,7 +174,10 @@ pub(crate) async fn launch_rpc_mgr<R: Runtime>(
         }
         drop(guards);
     })?;
-    Ok(Some((rpc_mgr, rpc_state_sender)))
+    Ok(Some(RpcProxySupport {
+        rpc_mgr,
+        rpc_state_sender,
+    }))
 }
 
 /// Backend function to implement an RPC listener: runs in a loop.
@@ -186,22 +187,9 @@ async fn run_rpc_listener<R: Runtime>(
     rpc_mgr: Arc<RpcMgr>,
 ) -> Result<()> {
     while let Some((stream, _addr, info)) = incoming.next().await.transpose()? {
-        // TODO RPC: Perhaps we should have rpcmgr hold the client reference?
-        // TODO RPC: We'll need to pass info (or part of it?) to rpc_mgr.
         debug!("Received incoming RPC connection from {}", &info.name);
 
-        match info.auth {
-            RpcAuth::None => {
-                // "None" auth works trivially; there's nothing to do.
-            }
-            _ => {
-                // TODO RPC: implement cookie auth, and reject other auth types earlier.
-                debug!("Dropping RPC connection; auth type is not supported");
-                continue;
-            }
-        }
-
-        let connection = rpc_mgr.new_connection();
+        let connection = rpc_mgr.new_connection(info.auth.clone());
         let (input, output) = stream.split();
 
         runtime.spawn(async {
@@ -214,6 +202,15 @@ async fn run_rpc_listener<R: Runtime>(
     Ok(())
 }
 
+/// Information passed to a SOCKS proxy or similar stream provider when running with RPC support.
+#[cfg_attr(feature = "experimental-api", visibility::make(pub))]
+pub(crate) struct RpcProxySupport {
+    /// An RPC manager to use for looking up objects as possible stream targets.
+    pub(crate) rpc_mgr: Arc<arti_rpcserver::RpcMgr>,
+    /// An RPCStateSender to use for registering the list of known proxy ports.
+    pub(crate) rpc_state_sender: RpcStateSender,
+}
+
 #[cfg(test)]
 mod test {
     // @@ begin test lint list maintained by maint/add_warning @@
