@@ -8,7 +8,8 @@ use clap::{ArgMatches, Args, FromArgMatches, Parser, Subcommand, ValueEnum};
 use tor_rtcompat::Runtime;
 
 use std::fs::OpenOptions;
-use std::io;
+use std::io::{self, Write};
+use std::str::FromStr;
 
 /// The hsc subcommands the arti CLI will be augmented with.
 #[derive(Parser, Debug)]
@@ -36,7 +37,6 @@ pub(crate) enum HscSubcommand {
 #[derive(Debug, Subcommand)]
 pub(crate) enum KeySubcommand {
     /// Get or generate a hidden service client key
-    /// Deprecated. Use key get instead.
     #[command(arg_required_else_help = true)]
     Get(GetKeyArgs),
 
@@ -93,7 +93,7 @@ enum GenerateKey {
 /// The common arguments of the key subcommands.
 #[derive(Debug, Clone, Args)]
 pub(crate) struct CommonArgs {
-    /// The type of key to rotate.
+    /// The type of the key.
     #[arg(
         long,
         default_value_t = KeyType::ServiceDiscovery,
@@ -101,9 +101,9 @@ pub(crate) struct CommonArgs {
     )]
     key_type: KeyType,
 
-    /// The .onion address of the hidden service
-    #[arg(long)]
-    onion_name: HsId,
+    /// Whether to write the prompt or not
+    #[arg(long, short, default_value_t = false)]
+    quiet: bool,
 }
 
 /// The common arguments of the key subcommands.
@@ -184,20 +184,18 @@ fn run_key(subcommand: KeySubcommand, client: &InertTorClient) -> Result<()> {
 
 /// Run the `hsc prepare-stealth-mode-key` subcommand.
 fn prepare_service_discovery_key(args: &GetKeyArgs, client: &InertTorClient) -> Result<()> {
+    let addr = get_onion_address(args.common.quiet)?;
     let key = match args.generate {
         GenerateKey::IfNeeded => {
             // TODO: consider using get_or_generate in generate_service_discovery_key
             client
-                .get_service_discovery_key(args.common.onion_name)?
+                .get_service_discovery_key(addr)?
                 .map(Ok)
                 .unwrap_or_else(|| {
-                    client.generate_service_discovery_key(
-                        KeystoreSelector::Primary,
-                        args.common.onion_name,
-                    )
+                    client.generate_service_discovery_key(KeystoreSelector::Primary, addr)
                 })?
         }
-        GenerateKey::No => match client.get_service_discovery_key(args.common.onion_name)? {
+        GenerateKey::No => match client.get_service_discovery_key(addr)? {
             Some(key) => key,
             None => {
                 return Err(anyhow!(
@@ -234,7 +232,6 @@ fn display_service_discovery_key(args: &KeygenArgs, key: &HsClientDescEncKey) ->
                         return Err(anyhow!("{filename} already exists. Move it, or rerun with --overwrite to overwrite it"));
                     }
                     _ => {
-                        // TODO maybe handle some other ErrorKinds
                         return Err(e)
                             .with_context(|| format!("could not write public key to {filename}"));
                     }
@@ -248,42 +245,36 @@ fn display_service_discovery_key(args: &KeygenArgs, key: &HsClientDescEncKey) ->
 
 /// Write the public part of `key` to `f`.
 fn write_public_key(mut f: impl io::Write, key: &HsClientDescEncKey) -> io::Result<()> {
-    write!(f, "{}", key)?;
+    writeln!(f, "{}", key)?;
     Ok(())
 }
 
 /// Run the `hsc rotate-key` subcommand.
 fn rotate_service_discovery_key(args: &RotateKeyArgs, client: &InertTorClient) -> Result<()> {
+    let addr = get_onion_address(args.common.quiet)?;
     if !args.force {
-        let msg = format!(
-            "rotate client restricted discovery key for {}?",
-            args.common.onion_name
-        );
+        let msg = format!("rotate client restricted discovery key for {}?", addr);
         if !prompt(&msg)? {
             return Ok(());
         }
     }
 
-    let key =
-        client.rotate_service_discovery_key(KeystoreSelector::default(), args.common.onion_name)?;
+    let key = client.rotate_service_discovery_key(KeystoreSelector::default(), addr)?;
 
     display_service_discovery_key(&args.keygen, &key)
 }
 
 /// Run the `hsc remove-key` subcommand.
 fn remove_service_discovery_key(args: &RemoveKeyArgs, client: &InertTorClient) -> Result<()> {
+    let addr = get_onion_address(args.common.quiet)?;
     if !args.force {
-        let msg = format!(
-            "remove client restricted discovery key for {}?",
-            args.common.onion_name
-        );
+        let msg = format!("remove client restricted discovery key for {}?", addr);
         if !prompt(&msg)? {
             return Ok(());
         }
     }
 
-    let _key =
-        client.remove_service_discovery_key(KeystoreSelector::default(), args.common.onion_name)?;
+    let _key = client.remove_service_discovery_key(KeystoreSelector::default(), addr)?;
 
     Ok(())
 }
@@ -317,3 +308,15 @@ fn prompt(msg: &str) -> Result<bool> {
         }
     }
 }
+
+/// Prompt the user for an onion address.
+fn get_onion_address(quiet: bool) -> Result<HsId, anyhow::Error> {
+    let mut addr = String::new();
+    if !quiet {
+        print!("Enter an onion address: ");
+        io::stdout().flush().map_err(|e| anyhow!(e))?;
+    };
+    io::stdin().read_line(&mut addr).map_err(|e| anyhow!(e))?;
+
+    HsId::from_str(addr.trim_end()).map_err(|e| anyhow!(e))
+}
