@@ -3,15 +3,12 @@
 
 use std::{
     fs::{File, Metadata, OpenOptions},
-    io::{self, Read, Write},
+    io,
     path::{Path, PathBuf},
 };
 
 use crate::{walk::PathType, Error, Mistrust, Result, Verifier};
 
-#[cfg(target_family = "unix")]
-use std::os::unix::fs::OpenOptionsExt;
-
 /// A directory whose access properties we have verified, along with accessor
 /// functions to access members of that directory.
 ///
@@ -84,6 +81,11 @@ impl CheckedDir {
         self.verifier().secure_dir(self.location.join(path))
     }
 
+    /// Create a new [`FileAccess`](crate::FileAccess) for reading or writing files within this directory.
+    pub fn file_access(&self) -> crate::FileAccess<'_> {
+        crate::FileAccess::from_checked_dir(self)
+    }
+
     /// Open a file within this CheckedDir, using a set of [`OpenOptions`].
     ///
     /// `path` must be a relative path to the new directory, containing no `..`
@@ -95,43 +97,7 @@ impl CheckedDir {
     /// always create it with mode `600`, regardless of any mode options set in
     /// `options`.
     pub fn open<P: AsRef<Path>>(&self, path: P, options: &OpenOptions) -> Result<File> {
-        let path = path.as_ref();
-        self.check_path(path)?;
-        let path = self.location.join(path);
-        if let Some(parent) = path.parent() {
-            self.verifier().check(parent)?;
-        }
-
-        #[allow(unused_mut)]
-        let mut options = options.clone();
-
-        #[cfg(target_family = "unix")]
-        {
-            // By default, create all files mode 600, no matter what
-            // OpenOptions said.
-
-            // TODO: Give some way to override this to 640 or 0644 if you
-            //    really want to.
-            options.mode(0o600);
-            // Don't follow symlinks out of the secured directory.
-            options.custom_flags(libc::O_NOFOLLOW);
-        }
-
-        let file = options
-            .open(&path)
-            .map_err(|e| Error::io(e, &path, "open file"))?;
-        let meta = file.metadata().map_err(|e| Error::inspecting(e, &path))?;
-
-        if let Some(error) = self
-            .verifier()
-            .check_one(path.as_path(), PathType::Content, &meta)
-            .into_iter()
-            .next()
-        {
-            Err(error)
-        } else {
-            Ok(file)
-        }
+        self.file_access().open(path, options)
     }
 
     /// List the contents of a directory within this [`CheckedDir`].
@@ -143,10 +109,7 @@ impl CheckedDir {
     /// The return value is an iterator as returned by [`std::fs::ReadDir`].  We
     /// _do not_ check any properties of the elements of this iterator.
     pub fn read_directory<P: AsRef<Path>>(&self, path: P) -> Result<std::fs::ReadDir> {
-        let path = path.as_ref();
-        self.check_path(path)?;
-        let path = self.location.join(path);
-        self.verifier().check(&path)?;
+        let path = self.verified_full_path(path.as_ref(), FullPathCheck::CheckPath)?;
 
         std::fs::read_dir(&path).map_err(|e| Error::io(e, path, "read directory"))
     }
@@ -159,17 +122,12 @@ impl CheckedDir {
     /// unmodifiable by any untrusted user, but we do not check any permissions
     /// on the file itself, since those are irrelevant to removing it.
     pub fn remove_file<P: AsRef<Path>>(&self, path: P) -> Result<()> {
-        let path = path.as_ref();
-        self.check_path(path)?;
-        let path = self.location.join(path);
         // We insist that the ownership and permissions on everything up to and
         // including the _parent_ of the path that we are removing have to be
         // correct.  (If it were otherwise, we could be tricked into removing
         // the wrong thing.)  But we don't care about the permissions on file we
         // are removing.
-        if let Some(parent) = path.parent() {
-            self.verifier().check(parent)?;
-        }
+        let path = self.verified_full_path(path.as_ref(), FullPathCheck::CheckParent)?;
 
         std::fs::remove_file(&path).map_err(|e| Error::io(e, path, "remove file"))
     }
@@ -201,12 +159,7 @@ impl CheckedDir {
     /// if it has any components that could take us outside of this directory,
     /// or if its contents are not UTF-8.
     pub fn read_to_string<P: AsRef<Path>>(&self, path: P) -> Result<String> {
-        let path = path.as_ref();
-        let mut file = self.open(path, OpenOptions::new().read(true))?;
-        let mut result = String::new();
-        file.read_to_string(&mut result)
-            .map_err(|e| Error::io(e, path, "read file"))?;
-        Ok(result)
+        self.file_access().read_to_string(path)
     }
 
     /// Read the contents of the file at `path` within this directory, as a
@@ -216,12 +169,7 @@ impl CheckedDir {
     /// or if it has any components that could take us outside of this
     /// directory.
     pub fn read<P: AsRef<Path>>(&self, path: P) -> Result<Vec<u8>> {
-        let path = path.as_ref();
-        let mut file = self.open(path, OpenOptions::new().read(true))?;
-        let mut result = Vec::new();
-        file.read_to_end(&mut result)
-            .map_err(|e| Error::io(e, path, "read file"))?;
-        Ok(result)
+        self.file_access().read(path)
     }
 
     /// Store `contents` into the file located at `path` within this directory.
@@ -245,26 +193,7 @@ impl CheckedDir {
         path: P,
         contents: C,
     ) -> Result<()> {
-        let path = path.as_ref();
-        self.check_path(path)?;
-
-        let tmp_name = path.with_extension("tmp");
-        let mut tmp_file = self.open(
-            &tmp_name,
-            OpenOptions::new().create(true).truncate(true).write(true),
-        )?;
-
-        // Write the data.
-        tmp_file
-            .write_all(contents.as_ref())
-            .map_err(|e| Error::io(e, &tmp_name, "write to file"))?;
-        // Flush and close.
-        drop(tmp_file);
-
-        // Replace the old file.
-        std::fs::rename(self.location.join(tmp_name), self.location.join(path))
-            .map_err(|e| Error::io(e, path, "replace file"))?;
-        Ok(())
+        self.file_access().write_and_replace(path, contents)
     }
 
     /// Return the [`Metadata`] of the file located at `path`.
@@ -282,12 +211,7 @@ impl CheckedDir {
     ///
     /// [^1]: the permissions are incorrect if the path is readable or writable by untrusted users
     pub fn metadata<P: AsRef<Path>>(&self, path: P) -> Result<Metadata> {
-        let path = path.as_ref();
-        self.check_path(path)?;
-        let path = self.location.join(path);
-        if let Some(parent) = path.parent() {
-            self.verifier().check(parent)?;
-        }
+        let path = self.verified_full_path(path.as_ref(), FullPathCheck::CheckParent)?;
 
         let meta = path
             .symlink_metadata()
@@ -328,6 +252,8 @@ impl CheckedDir {
 
     /// Helper: Make sure that the path `p` is a relative path that can be
     /// guaranteed to stay within this directory.
+    ///
+    /// (Specifically, we reject absolute paths, ".." items, and Windows path prefixes.)
     fn check_path(&self, p: &Path) -> Result<()> {
         use std::path::Component;
         // This check should be redundant, but let's be certain.
@@ -346,6 +272,34 @@ impl CheckedDir {
 
         Ok(())
     }
+
+    /// Check whether `p` is a valid relative path within this directory,
+    /// verify its permissions or the permissions of its parent, depending on `check_type`,
+    /// and return an absolute path for `p`.
+    pub(crate) fn verified_full_path(
+        &self,
+        p: &Path,
+        check_type: FullPathCheck,
+    ) -> Result<PathBuf> {
+        self.check_path(p)?;
+        let full_path = self.location.join(p);
+        let to_verify: &Path = match check_type {
+            FullPathCheck::CheckPath => full_path.as_ref(),
+            FullPathCheck::CheckParent => full_path.parent().unwrap_or_else(|| full_path.as_ref()),
+        };
+        self.verifier().check(to_verify)?;
+
+        Ok(full_path)
+    }
+}
+
+/// Type argument for [`CheckedDir::verified_full_path`].
+#[derive(Clone, Copy, Debug)]
+pub(crate) enum FullPathCheck {
+    /// Check all elements of the path, including the final element.
+    CheckPath,
+    /// Check all elements of the path, not including the final element.
+    CheckParent,
 }
 
 #[cfg(test)]
@@ -607,7 +561,7 @@ mod test {
 
     #[test]
     #[cfg(target_family = "unix")]
-    fn metadata_symlink() {
+    fn access_symlink() {
         use crate::testing::LinkType;
 
         let d = Dir::new();
@@ -633,5 +587,14 @@ mod test {
             matches!(e, Error::Io { ref err, .. } if err.to_string().contains("is a symlink")),
             "{e:?}"
         );
+
+        // Open returns an error if called on a symlink.
+        let e = sd
+            .open("f1-link", OpenOptions::new().read(true))
+            .unwrap_err();
+        assert!(
+            matches!(e, Error::Io { ref err, .. } if err.to_string().contains("symbolic")), // Error is ELOOP.
+            "{e:?}"
+        );
     }
 }
