@@ -11,8 +11,9 @@ use std::{
 };
 
 use crate::{
-    build::CircuitBuilder, mgr::AbstractCircBuilder, timeouts, AbstractCirc, CircMgr, CircMgrInner,
-    Error, Result,
+    build::{onion_circparams_from_netparams, CircuitBuilder},
+    mgr::AbstractCircBuilder,
+    timeouts, AbstractCirc, CircMgr, CircMgrInner, Error, Result,
 };
 use futures::{task::SpawnExt, StreamExt, TryFutureExt};
 use once_cell::sync::OnceCell;
@@ -64,34 +65,34 @@ pub enum HsCircKind {
 }
 
 impl HsCircKind {
-    /// Return the [`HsCircStubKind`] needed to build this type of circuit.
-    fn stub_kind(&self) -> HsCircStubKind {
+    /// Return the [`HsCircStemKind`] needed to build this type of circuit.
+    fn stem_kind(&self) -> HsCircStemKind {
         match self {
-            HsCircKind::ClientRend | HsCircKind::SvcIntro => HsCircStubKind::Short,
+            HsCircKind::ClientRend | HsCircKind::SvcIntro => HsCircStemKind::Naive,
             HsCircKind::SvcHsDir => {
-                // TODO: we might want this to be EXTENDED
-                HsCircStubKind::Short
+                // TODO: we might want this to be GUARDED
+                HsCircStemKind::Naive
             }
             HsCircKind::SvcRend | HsCircKind::ClientHsDir | HsCircKind::ClientIntro => {
-                HsCircStubKind::Extended
+                HsCircStemKind::Guarded
             }
         }
     }
 }
 
-/// A hidden service circuit stub.
+/// A hidden service circuit stem.
 ///
 /// This represents a hidden service circuit that has not yet been extended to a target.
 ///
-/// See [HsCircStubKind].
-pub(crate) struct HsCircStub<C: AbstractCirc> {
+/// See [HsCircStemKind].
+pub(crate) struct HsCircStem<C: AbstractCirc> {
     /// The circuit.
     pub(crate) circ: Arc<C>,
-    /// Whether the circuit is SHORT or EXTENDED
-    pub(crate) kind: HsCircStubKind,
+    /// Whether the circuit is NAIVE  or GUARDED.
+    pub(crate) kind: HsCircStemKind,
 }
 
-impl<C: AbstractCirc> HsCircStub<C> {
+impl<C: AbstractCirc> HsCircStem<C> {
     /// Whether this circuit satisfies _all_ the [`HsCircPrefs`].
     ///
     /// Returns `false` if any of the `prefs` are not satisfied.
@@ -105,7 +106,7 @@ impl<C: AbstractCirc> HsCircStub<C> {
     }
 }
 
-impl<C: AbstractCirc> Deref for HsCircStub<C> {
+impl<C: AbstractCirc> Deref for HsCircStem<C> {
     type Target = Arc<C>;
 
     fn deref(&self) -> &Self::Target {
@@ -113,78 +114,78 @@ impl<C: AbstractCirc> Deref for HsCircStub<C> {
     }
 }
 
-impl<C: AbstractCirc> HsCircStub<C> {
-    /// Check if this circuit stub is of the specified `kind`
+impl<C: AbstractCirc> HsCircStem<C> {
+    /// Check if this circuit stem is of the specified `kind`
     /// or can be extended to become that kind.
     ///
-    /// Returns `true` if this `HsCircStub`'s kind is equal to `other`,
-    /// or if its kind is [`Short`](HsCircStubKind::Short)
-    /// and `other` is [`Extended`](HsCircStubKind::Extended).
-    pub(crate) fn can_become(&self, other: HsCircStubKind) -> bool {
-        use HsCircStubKind::*;
+    /// Returns `true` if this `HsCircStem`'s kind is equal to `other`,
+    /// or if its kind is [`Naive`](HsCircStemKind::Naive)
+    /// and `other` is [`Guarded`](HsCircStemKind::Guarded).
+    pub(crate) fn can_become(&self, other: HsCircStemKind) -> bool {
+        use HsCircStemKind::*;
 
         match (self.kind, other) {
-            (Short, Short) | (Extended, Extended) | (Short, Extended) => true,
-            (Extended, Short) => false,
+            (Naive, Naive) | (Guarded, Guarded) | (Naive, Guarded) => true,
+            (Guarded, Naive) => false,
         }
     }
 }
 
 #[allow(rustdoc::private_intra_doc_links)]
-/// A kind of hidden service circuit stub.
+/// A kind of hidden service circuit stem.
 ///
 /// See [hspath](crate::path::hspath) docs for more information.
 ///
-/// The structure of a stub circuit depends on whether vanguards are enabled:
+/// The structure of a circuit stem depends on whether vanguards are enabled:
 ///
 ///   * with vanguards disabled:
 ///      ```text
-///         SHORT    = G -> M -> M
-///         EXTENDED = G -> M -> M
+///         NAIVE   = G -> M -> M
+///         GUARDED = G -> M -> M
 ///      ```
 ///
 ///   * with lite vanguards enabled:
 ///      ```text
-///         SHORT    = G -> L2 -> M
-///         EXTENDED = G -> L2 -> M
+///         NAIVE   = G -> L2 -> M
+///         GUARDED = G -> L2 -> M
 ///      ```
 ///
 ///   * with full vanguards enabled:
 ///      ```text
-///         SHORT    = G -> L2 -> L3
-///         EXTENDED = G -> L2 -> L3 -> M
+///         NAIVE    = G -> L2 -> L3
+///         GUARDED = G -> L2 -> L3 -> M
 ///      ```
 #[derive(Copy, Clone, Debug, PartialEq, derive_more::Display)]
 #[non_exhaustive]
-pub(crate) enum HsCircStubKind {
-    /// A short stub circuit.
+pub(crate) enum HsCircStemKind {
+    /// A naive circuit stem.
     ///
     /// Used for building circuits to a final hop that an adversary cannot easily control,
     /// for example if the final hop is is randomly chosen by us.
-    #[display("SHORT")]
-    Short,
-    /// An extended stub circuit.
+    #[display("NAIVE")]
+    Naive,
+    /// An guarded circuit stem.
     ///
     /// Used for building circuits to a final hop that an adversary can easily control,
     /// for example if the final hop is not chosen by us.
-    #[display("EXTENDED")]
-    Extended,
+    #[display("GUARDED")]
+    Guarded,
 }
 
-impl HsCircStubKind {
+impl HsCircStemKind {
     /// Return the number of hops this `HsCircKind` ought to have when using the specified
     /// [`VanguardMode`].
     pub(crate) fn num_hops(&self, mode: VanguardMode) -> StdResult<usize, Bug> {
-        use HsCircStubKind::*;
+        use HsCircStemKind::*;
         use VanguardMode::*;
 
         let len = match (mode, self) {
             #[cfg(all(feature = "vanguards", feature = "hs-common"))]
             (Lite, _) => 3,
             #[cfg(all(feature = "vanguards", feature = "hs-common"))]
-            (Full, Short) => 3,
+            (Full, Naive) => 3,
             #[cfg(all(feature = "vanguards", feature = "hs-common"))]
-            (Full, Extended) => 4,
+            (Full, Guarded) => 4,
             (Disabled, _) => 3,
             (_, _) => {
                 return Err(internal!("Unsupported vanguard mode {mode}"));
@@ -350,21 +351,21 @@ impl<B: AbstractCircBuilder<R> + 'static, R: Runtime> HsCircPoolInner<B, R> {
         // For rendezvous points, clients use 3-hop circuits.
         // Note that we aren't using any special rules for the last hop here; we
         // are relying on the fact that:
-        //   * all suitable middle relays that we use in these stub circuits are
+        //   * all suitable middle relays that we use in these circuit stems are
         //     suitable renedezvous points, and
         //   * the weighting rules for selecting rendezvous points are the same
         //     as those for selecting an arbitrary middle relay.
         let circ = self
-            .take_or_launch_stub_circuit::<OwnedCircTarget>(netdir, None, HsCircStubKind::Extended)
+            .take_or_launch_stem_circuit::<OwnedCircTarget>(netdir, None, HsCircStemKind::Guarded)
             .await?;
 
         #[cfg(all(feature = "vanguards", feature = "hs-common"))]
         if matches!(
             self.vanguard_mode(),
             VanguardMode::Full | VanguardMode::Lite
-        ) && circ.kind != HsCircStubKind::Extended
+        ) && circ.kind != HsCircStemKind::Guarded
         {
-            return Err(internal!("wanted a EXTENDED circuit, but got SHORT?!").into());
+            return Err(internal!("wanted a GUARDED circuit, but got NAIVE?!").into());
         }
 
         let path = circ.path_ref();
@@ -382,7 +383,7 @@ impl<B: AbstractCircBuilder<R> + 'static, R: Runtime> HsCircPoolInner<B, R> {
                     // every relay in the circuit is listed.
                     //
                     // TODO: Still, it's an ugly place in our API; maybe we should return the last hop
-                    // from take_or_launch_stub_circuit()?  But in many cases it won't be needed...
+                    // from take_or_launch_stem_circuit()?  But in many cases it won't be needed...
                     None => Err(internal!("Got circuit with unknown last hop!?").into()),
                 }
             }
@@ -404,7 +405,7 @@ impl<B: AbstractCircBuilder<R> + 'static, R: Runtime> HsCircPoolInner<B, R> {
             return Err(bad_api_usage!("get_or_launch_specific with ClientRend circuit!?").into());
         }
 
-        let wanted_kind = kind.stub_kind();
+        let wanted_kind = kind.stem_kind();
 
         // For most* of these circuit types, we want to build our circuit with
         // an extra hop, since the target hop is under somebody else's control.
@@ -415,7 +416,7 @@ impl<B: AbstractCircBuilder<R> + 'static, R: Runtime> HsCircPoolInner<B, R> {
 
         // Get an unfinished circuit that's compatible with our target.
         let circ = self
-            .take_or_launch_stub_circuit(netdir, Some(&target), wanted_kind)
+            .take_or_launch_stem_circuit(netdir, Some(&target), wanted_kind)
             .await?;
 
         #[cfg(all(feature = "vanguards", feature = "hs-common"))]
@@ -425,20 +426,20 @@ impl<B: AbstractCircBuilder<R> + 'static, R: Runtime> HsCircPoolInner<B, R> {
         ) && circ.kind != wanted_kind
         {
             return Err(internal!(
-                "take_or_launch_stub_circuit() returned {:?}, but we need {wanted_kind:?}",
+                "take_or_launch_stem_circuit() returned {:?}, but we need {wanted_kind:?}",
                 circ.kind
             )
             .into());
         }
 
-        let params = crate::DirInfo::from(netdir).circ_params();
+        let params = onion_circparams_from_netparams(netdir.params())?;
         self.extend_circ(circ, params, target).await
     }
 
     /// Try to extend a circuit to the specified target hop.
     async fn extend_circ<T>(
         &self,
-        circ: HsCircStub<B::Circ>,
+        circ: HsCircStem<B::Circ>,
         params: CircParameters,
         target: T,
     ) -> Result<Arc<B::Circ>>
@@ -490,18 +491,18 @@ impl<B: AbstractCircBuilder<R> + 'static, R: Runtime> HsCircPoolInner<B, R> {
 
     /// Take and return a circuit from our pool suitable for being extended to `avoid_target`.
     ///
-    /// If vanguards are enabled, this will try to build a circuit stub of the specified
-    /// [`HsCircStubKind`].
+    /// If vanguards are enabled, this will try to build a circuit stem of the specified
+    /// [`HsCircStemKind`].
     ///
     /// If vanguards are disabled, `kind` is unused.
     ///
     /// If there is no such circuit, build and return a new one.
-    async fn take_or_launch_stub_circuit<T>(
+    async fn take_or_launch_stem_circuit<T>(
         &self,
         netdir: &NetDir,
         avoid_target: Option<&T>,
-        kind: HsCircStubKind,
-    ) -> Result<HsCircStub<B::Circ>>
+        kind: HsCircStemKind,
+    ) -> Result<HsCircStem<B::Circ>>
     where
         // TODO #504: It would be better if this were a type that had to include
         // family info.
@@ -511,7 +512,7 @@ impl<B: AbstractCircBuilder<R> + 'static, R: Runtime> HsCircPoolInner<B, R> {
         trace!(
             vanguards=%vanguard_mode,
             kind=%kind,
-            "selecting HS circuit stub"
+            "selecting HS circuit stem"
         );
 
         // First, look for a circuit that is already built, if any is suitable.
@@ -530,7 +531,7 @@ impl<B: AbstractCircBuilder<R> + 'static, R: Runtime> HsCircPoolInner<B, R> {
         let found_usable_circ = {
             let mut inner = self.inner.lock().expect("lock poisoned");
 
-            let restrictions = |circ: &HsCircStub<B::Circ>| {
+            let restrictions = |circ: &HsCircStem<B::Circ>| {
                 // If vanguards are enabled, we no longer apply same-family or same-subnet
                 // restrictions, and we allow the guard to appear as either of the last
                 // two hope of the circuit.
@@ -553,13 +554,13 @@ impl<B: AbstractCircBuilder<R> + 'static, R: Runtime> HsCircPoolInner<B, R> {
 
             #[cfg(all(feature = "vanguards", feature = "hs-common"))]
             if matches!(vanguard_mode, VanguardMode::Full | VanguardMode::Lite) {
-                prefs.preferred_stub_kind(kind);
+                prefs.preferred_stem_kind(kind);
             }
 
             let found_usable_circ =
                 inner
                     .pool
-                    .take_one_where(&mut rand::thread_rng(), restrictions, &prefs);
+                    .take_one_where(&mut rand::rng(), restrictions, &prefs);
 
             // Tell the background task to fire immediately if we have very few circuits
             // circuits left, or if we found nothing.
@@ -574,7 +575,7 @@ impl<B: AbstractCircBuilder<R> + 'static, R: Runtime> HsCircPoolInner<B, R> {
         // Return the circuit we found before, if any.
         if let Some(circuit) = found_usable_circ {
             let circuit = self
-                .maybe_extend_stub_circuit(netdir, circuit, avoid_target, kind)
+                .maybe_extend_stem_circuit(netdir, circuit, avoid_target, kind)
                 .await?;
             self.ensure_suitable_circuit(&circuit, avoid_target, kind)?;
             return Ok(circuit);
@@ -593,32 +594,32 @@ impl<B: AbstractCircBuilder<R> + 'static, R: Runtime> HsCircPoolInner<B, R> {
 
         self.ensure_suitable_circuit(&circ, avoid_target, kind)?;
 
-        Ok(HsCircStub { circ, kind })
+        Ok(HsCircStem { circ, kind })
     }
 
     /// Return a circuit of the specified `kind`, built from `circuit`.
-    async fn maybe_extend_stub_circuit<T>(
+    async fn maybe_extend_stem_circuit<T>(
         &self,
         netdir: &NetDir,
-        circuit: HsCircStub<B::Circ>,
+        circuit: HsCircStem<B::Circ>,
         avoid_target: Option<&T>,
-        kind: HsCircStubKind,
-    ) -> Result<HsCircStub<B::Circ>>
+        kind: HsCircStemKind,
+    ) -> Result<HsCircStem<B::Circ>>
     where
         T: CircTarget + std::marker::Sync,
     {
         match self.vanguard_mode() {
             #[cfg(all(feature = "vanguards", feature = "hs-common"))]
             VanguardMode::Full => {
-                // SHORT circuit stubs need to be extended by one hop to become EXTENDED stubs
+                // NAIVE circuit stems need to be extended by one hop to become GUARDED stems
                 // if we're using full vanguards.
                 self.extend_full_vanguards_circuit(netdir, circuit, avoid_target, kind)
                     .await
             }
             _ => {
-                let HsCircStub { circ, kind: _ } = circuit;
+                let HsCircStem { circ, kind: _ } = circuit;
 
-                Ok(HsCircStub { circ, kind })
+                Ok(HsCircStem { circ, kind })
             }
         }
     }
@@ -628,20 +629,20 @@ impl<B: AbstractCircBuilder<R> + 'static, R: Runtime> HsCircPoolInner<B, R> {
     async fn extend_full_vanguards_circuit<T>(
         &self,
         netdir: &NetDir,
-        circuit: HsCircStub<B::Circ>,
+        circuit: HsCircStem<B::Circ>,
         avoid_target: Option<&T>,
-        kind: HsCircStubKind,
-    ) -> Result<HsCircStub<B::Circ>>
+        kind: HsCircStemKind,
+    ) -> Result<HsCircStem<B::Circ>>
     where
         T: CircTarget + std::marker::Sync,
     {
         match (circuit.kind, kind) {
-            (HsCircStubKind::Short, HsCircStubKind::Extended) => {
-                debug!("Wanted EXTENDED circuit, but got SHORT; extending by 1 hop...");
-                let params = CircParameters::default();
+            (HsCircStemKind::Naive, HsCircStemKind::Guarded) => {
+                debug!("Wanted GUARDED circuit, but got NAIVE; extending by 1 hop...");
+                let params = crate::build::onion_circparams_from_netparams(netdir.params())?;
                 let circ_path = circuit.circ.path_ref();
 
-                // A SHORT circuit is a 3-hop circuit.
+                // A NAIVE circuit is a 3-hop circuit.
                 debug_assert_eq!(circ_path.hops().len(), 3);
 
                 let target_exclusion = if let Some(target) = &avoid_target {
@@ -660,21 +661,21 @@ impl<B: AbstractCircBuilder<R> + 'static, R: Runtime> HsCircPoolInner<B, R> {
                     &hops,
                     netdir,
                     &target_exclusion,
-                    &mut rand::thread_rng(),
+                    &mut rand::rng(),
                 )?;
 
-                // Since full vanguards are enabled and the circuit we got is SHORT,
-                // we need to extend it by another hop to make it EXTENDED before returning it
+                // Since full vanguards are enabled and the circuit we got is NAIVE,
+                // we need to extend it by another hop to make it GUARDED before returning it
                 let circ = self.extend_circ(circuit, params, extra_hop).await?;
 
-                Ok(HsCircStub { circ, kind })
+                Ok(HsCircStem { circ, kind })
             }
-            (HsCircStubKind::Extended, HsCircStubKind::Short) => {
-                Err(internal!("wanted a SHORT circuit, but got EXTENDED?!").into())
+            (HsCircStemKind::Guarded, HsCircStemKind::Naive) => {
+                Err(internal!("wanted a NAIVE circuit, but got GUARDED?!").into())
             }
             _ => {
                 trace!("Wanted {kind} circuit, got {}", circuit.kind);
-                // Nothing to do: the circuit stub we got is of the kind we wanted
+                // Nothing to do: the circuit stem we got is of the kind we wanted
                 Ok(circuit)
             }
         }
@@ -685,7 +686,7 @@ impl<B: AbstractCircBuilder<R> + 'static, R: Runtime> HsCircPoolInner<B, R> {
         &self,
         circ: &Arc<B::Circ>,
         target: Option<&T>,
-        kind: HsCircStubKind,
+        kind: HsCircStemKind,
     ) -> StdResult<(), Bug>
     where
         T: CircTarget + std::marker::Sync,
@@ -700,7 +701,7 @@ impl<B: AbstractCircBuilder<R> + 'static, R: Runtime> HsCircPoolInner<B, R> {
     fn ensure_circuit_length_valid(
         &self,
         circ: &Arc<B::Circ>,
-        kind: HsCircStubKind,
+        kind: HsCircStemKind,
     ) -> StdResult<(), Bug> {
         let circ_path_len = circ.path_ref().n_hops();
         let mode = self.vanguard_mode();
@@ -802,7 +803,7 @@ impl<B: AbstractCircBuilder<R> + 'static, R: Runtime> HsCircPoolInner<B, R> {
 /// `target`.
 fn circuit_compatible_with_target<C: AbstractCirc>(
     netdir: &NetDir,
-    circ: &HsCircStub<C>,
+    circ: &HsCircStem<C>,
     exclude_target: &RelayExclusion,
 ) -> bool {
     // NOTE, TODO #504:
@@ -821,12 +822,12 @@ fn circuit_compatible_with_target<C: AbstractCirc>(
 /// Return true if we can extend a pre-built vanguards circuit `circ` to `target`.
 ///
 /// We require that the circuit is open, that it can become the specified
-/// kind of [`HsCircStub`], that every hop in the circuit is listed in `netdir`,
+/// kind of [`HsCircStem`], that every hop in the circuit is listed in `netdir`,
 /// and that the last two hops are different from the specified target.
 fn vanguards_circuit_compatible_with_target<C: AbstractCirc, T>(
     netdir: &NetDir,
-    circ: &HsCircStub<C>,
-    kind: HsCircStubKind,
+    circ: &HsCircStem<C>,
+    kind: HsCircStemKind,
     avoid_target: Option<&T>,
 ) -> bool
 where
@@ -858,7 +859,7 @@ where
 /// We require that the circuit is open, that every hop  in the circuit is
 /// listed in `netdir`, and that `relay_okay` returns true for every hop on the
 /// circuit.
-fn circuit_still_useable<C, F>(netdir: &NetDir, circ: &HsCircStub<C>, relay_okay: F) -> bool
+fn circuit_still_useable<C, F>(netdir: &NetDir, circ: &HsCircStem<C>, relay_okay: F) -> bool
 where
     C: AbstractCirc,
     F: Fn(&Relay<'_>) -> bool,
@@ -914,9 +915,9 @@ async fn launch_hs_circuits_as_needed<B: AbstractCircBuilder<R> + 'static, R: Ru
 
         if n_to_launch > 0 {
             debug!(
-                "launching {} SHORT and {} EXTENDED circuits",
-                circs_to_launch.stub(),
-                circs_to_launch.ext_stub()
+                "launching {} NAIVE  and {} GUARDED circuits",
+                circs_to_launch.stem(),
+                circs_to_launch.guarded_stem()
             );
         }
 
@@ -947,7 +948,7 @@ async fn launch_hs_circuits_as_needed<B: AbstractCircBuilder<R> + 'static, R: Ru
                 {
                     Ok(circ) => {
                         let kind = for_launch.kind();
-                        let circ = HsCircStub { circ, kind };
+                        let circ = HsCircStem { circ, kind };
                         pool.inner.lock().expect("poisoned lock").pool.insert(circ);
                         trace!("successfully launched {kind} circuit");
                         for_launch.note_circ_launched();
@@ -1021,6 +1022,8 @@ mod test {
     #[cfg(all(feature = "vanguards", feature = "hs-common"))]
     use tor_guardmgr::VanguardConfigBuilder;
     use tor_guardmgr::VanguardMode;
+    use tor_memquota::ArcMemoryQuotaTrackerExt as _;
+    use tor_proto::memquota::ToplevelAccount;
     use tor_rtmock::MockRuntime;
 
     use super::*;
@@ -1036,6 +1039,7 @@ mod test {
             &Default::default(),
             tor_chanmgr::Dormancy::Dormant,
             &Default::default(),
+            ToplevelAccount::new_noop(),
         );
         let guardmgr = tor_guardmgr::GuardMgr::new(
             runtime.clone(),
@@ -1061,7 +1065,7 @@ mod test {
             tor_persist::TestingStateMgr::new(),
             &runtime,
             Arc::new(chanmgr),
-            guardmgr,
+            &guardmgr,
         )
         .unwrap()
         .into()
