@@ -50,18 +50,18 @@ mod macros;
 mod ssh;
 mod traits;
 
-pub use certs::CertData;
+pub use certs::{CertData, ParsedEd25519Cert, ValidatedEd25519Cert};
 pub use err::Error;
 pub use key_type::{CertType, KeyType, KeystoreItemType};
 pub use ssh::{SshKeyAlgorithm, SshKeyData};
 pub use traits::{
-    EncodableItem, InvalidCertError, Keygen, KeygenRng, KeystoreItem, ToEncodableCert,
+    EncodableItem, InvalidCertError, ItemType, Keygen, KeygenRng, KeystoreItem, ToEncodableCert,
     ToEncodableKey,
 };
 
-// Note: we use EncodedEd25519Cert in our public API here,
-// so let's reexport it for convenience...
-pub use tor_cert::EncodedEd25519Cert;
+// Note: we use various tor-cert types in our public API,
+// so let's reexport them for convenience...
+pub use tor_cert::{Ed25519Cert, EncodedEd25519Cert, KeyUnknownCert};
 
 // Needed to export our derive_deftly macros.
 #[doc(hidden)]
@@ -74,4 +74,4 @@ pub use macros::deps as macro_deps;
 pub type Result<T> = std::result::Result<T, Error>;
 
 /// A type-erased key. Used by the tor-keymgr.
-pub type ErasedKey = Box<dyn traits::EncodableItem>;
+pub type ErasedKey = Box<dyn traits::ItemType>;
