@@ -34,11 +34,13 @@
 #![deny(clippy::unnecessary_wraps)]
 #![warn(clippy::unseparated_literal_suffix)]
 #![deny(clippy::unwrap_used)]
+#![deny(clippy::mod_module_files)]
 #![allow(clippy::let_unit_value)] // This can reasonably be done for explicitness
 #![allow(clippy::uninlined_format_args)]
 #![allow(clippy::significant_drop_in_scrutinee)] // arti/-/merge_requests/588/#note_2812945
 #![allow(clippy::result_large_err)] // temporary workaround for arti#587
 #![allow(clippy::needless_raw_string_hashes)] // complained-about code is fine, often best
+#![allow(clippy::needless_lifetimes)] // See arti#1765
 //! <!-- @@ end lint list maintained by maint/add_warning @@ -->
 
 mod err;
@@ -56,6 +58,15 @@ pub use handshake::proxy::SocksProxyHandshake;
 #[cfg_attr(docsrs, doc(cfg(feature = "client-handshake")))]
 pub use handshake::client::SocksClientHandshake;
 
+#[cfg(any(feature = "proxy-handshake", feature = "client-handshake"))]
+#[cfg_attr(
+    docsrs,
+    doc(cfg(any(feature = "proxy-handshake", feature = "client-handshake")))
+)]
+pub use handshake::framework::{
+    Buffer, Finished, Handshake, NextStep, PreciseReads, ReadPrecision, RecvStep,
+};
+
 #[deprecated(since = "0.5.2", note = "Use SocksProxyHandshake instead.")]
 #[cfg(feature = "proxy-handshake")]
 #[cfg_attr(docsrs, doc(cfg(feature = "proxy-handshake")))]
